part of '../calendar_date_picker2.dart';

T? _ambiguate<T>(T? value) => value;

class _CalendarView extends StatefulWidget {
  /// Creates a month picker.
  const _CalendarView({
    required this.config,
    required this.initialMonth,
    required this.selectedDates,
    required this.onChanged,
    required this.onDisplayedMonthChanged,
    Key? key,
  }) : super(key: key);

  /// The calendar configurations
  final CalendarDatePicker2Config config;

  /// The initial month to display.
  final DateTime initialMonth;

  /// The currently selected dates.
  ///
  /// Selected dates are highlighted in the picker.
  final List<DateTime?> selectedDates;

  /// Called when the user picks a day.
  final ValueChanged<DateTime> onChanged;

  /// Called when the user navigates to a new month.
  final ValueChanged<DateTime> onDisplayedMonthChanged;

  @override
  _CalendarViewState createState() => _CalendarViewState();
}

class _CalendarViewState extends State<_CalendarView> {
  final GlobalKey _pageViewKey = GlobalKey();
  late DateTime _currentMonth;
  late PageController _pageController;
  late MaterialLocalizations _localizations;
  late TextDirection _textDirection;
  Map<ShortcutActivator, Intent>? _shortcutMap;
  Map<Type, Action<Intent>>? _actionMap;
  late FocusNode _dayGridFocus;
  DateTime? _focusedDay;

  final List<String> list = <String>[
    'Enero',
    'Febrero',
    'Marzo',
    'Abril',
    'Mayo',
    'Junio',
    'Julio',
    'Agosto',
    'Septiembre',
    'Octubre',
    'Noviembre',
    'Diciembre'
  ];
  String dropdownValue = "Enero";
  final List<int> anios = [1899];
  var dropAnio = DateTime.now().year;
  // int dropAnio = DateTime.now().year;

  @override
  void initState() {
    super.initState();
    _currentMonth = widget.initialMonth;
    _pageController = PageController(
        initialPage:
            DateUtils.monthDelta(widget.config.firstDate, _currentMonth));
    _shortcutMap = const <ShortcutActivator, Intent>{
      SingleActivator(LogicalKeyboardKey.arrowLeft):
          DirectionalFocusIntent(TraversalDirection.left),
      SingleActivator(LogicalKeyboardKey.arrowRight):
          DirectionalFocusIntent(TraversalDirection.right),
      SingleActivator(LogicalKeyboardKey.arrowDown):
          DirectionalFocusIntent(TraversalDirection.down),
      SingleActivator(LogicalKeyboardKey.arrowUp):
          DirectionalFocusIntent(TraversalDirection.up),
    };
    _actionMap = <Type, Action<Intent>>{
      NextFocusIntent:
          CallbackAction<NextFocusIntent>(onInvoke: _handleGridNextFocus),
      PreviousFocusIntent: CallbackAction<PreviousFocusIntent>(
          onInvoke: _handleGridPreviousFocus),
      DirectionalFocusIntent: CallbackAction<DirectionalFocusIntent>(
          onInvoke: _handleDirectionFocus),
    };
    _dayGridFocus = FocusNode(debugLabel: 'Day Grid');
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _localizations = MaterialLocalizations.of(context);
    _textDirection = Directionality.of(context);
  }

  @override
  void didUpdateWidget(_CalendarView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.initialMonth != oldWidget.initialMonth &&
        widget.initialMonth != _currentMonth) {
      // We can't interrupt this widget build with a scroll, so do it next frame
      // Add workaround to fix Flutter 3.0.0 compiler issue
      // https://github.com/flutter/flutter/issues/103561#issuecomment-1125512962
      // https://github.com/flutter/website/blob/3e6d87f13ad2a8dd9cf16081868cc3b3794abb90/src/development/tools/sdk/release-notes/release-notes-3.0.0.md#your-code
      _ambiguate(WidgetsBinding.instance)!.addPostFrameCallback(
        (Duration timeStamp) => _showMonth(widget.initialMonth, jump: true),
      );
    }
  }

  @override
  void dispose() {
    _pageController.dispose();
    _dayGridFocus.dispose();
    super.dispose();
  }

  void _handleDateSelected(DateTime selectedDate) {
    _focusedDay = selectedDate;
    widget.onChanged(selectedDate);
  }

  void _handleMonthPageChanged(int monthPage) {
    setState(() {
      final DateTime monthDate =
          DateUtils.addMonthsToMonthDate(widget.config.firstDate, monthPage);
      if (!DateUtils.isSameMonth(_currentMonth, monthDate)) {
        _currentMonth = DateTime(monthDate.year, monthDate.month);
        widget.onDisplayedMonthChanged(_currentMonth);
        if (_focusedDay != null &&
            !DateUtils.isSameMonth(_focusedDay, _currentMonth)) {
          // We have navigated to a new month with the grid focused, but the
          // focused day is not in this month. Choose a new one trying to keep
          // the same day of the month.
          _focusedDay = _focusableDayForMonth(_currentMonth, _focusedDay!.day);
        }
        SemanticsService.announce(
          _localizations.formatMonthYear(_currentMonth),
          _textDirection,
        );
      }
    });
  }

  /// Returns a focusable date for the given month.
  ///
  /// If the preferredDay is available in the month it will be returned,
  /// otherwise the first selectable day in the month will be returned. If
  /// no dates are selectable in the month, then it will return null.
  DateTime? _focusableDayForMonth(DateTime month, int preferredDay) {
    final int daysInMonth = DateUtils.getDaysInMonth(month.year, month.month);

    // Can we use the preferred day in this month?
    if (preferredDay <= daysInMonth) {
      final DateTime newFocus = DateTime(month.year, month.month, preferredDay);
      if (_isSelectable(newFocus)) return newFocus;
    }

    // Start at the 1st and take the first selectable date.
    for (int day = 1; day <= daysInMonth; day++) {
      final DateTime newFocus = DateTime(month.year, month.month, day);
      if (_isSelectable(newFocus)) return newFocus;
    }
    return null;
  }
   /// Navigate to the previous month.
  void _handlePreviousMonth() {
    if (!_isDisplayingFirstMonth) {
      
      // dropAnio = _currentMonth.year;
      _pageController.previousPage(
        duration: _monthScrollDuration,
        curve: Curves.ease,
      );
      dropdownValue = obtenerIntMes(_currentMonth.month);
      // dropAnio = _currentMonth.year;
      // print(_currentMonth.month);
      if (_currentMonth.month == 1) {
        dropAnio = _currentMonth.year-1;
      }
      
      
    }
  }
  /// Navigate to the next month.
  void _handleNextMonth() {
    
    if (!_isDisplayingLastMonth) {
      
      // dropAnio = _currentMonth.year;
      _pageController.nextPage(
        duration: _monthScrollDuration,
        curve: Curves.ease,
      );
      // _showMonth()
      dropdownValue = obtenerIntMes(_currentMonth.month);
      dropAnio = _currentMonth.year;
      // print(_currentMonth.month);
      if (_currentMonth.month == 12) {
        dropAnio = _currentMonth.year+1;
      }
      // if (_currentMonth.month == 12) {
      //   dropAnio = _currentMonth.year;
      // }
      // // 
      // print(_currentMonth.month);
      // _showMonth(DateTime(dropAnio,_currentMonth.month),jump: true);
      // if(_currentMonth.month ==12){
      //   dropAnio = dropAnio+1;
      // }
      
    }
  }

  String obtenerIntMes(int month) {
    switch (month) {
      case 1:
        return "Enero";
      case 2:
        return "Febrero";
      case 3:
        return "Marzo";
      case 4:
        return "Abril";
      case 5:
        return "Mayo";
      case 6:
        return "Junio";
      case 7:
        return "Julio";
      case 8:
        return "Agosto";
      case 9:
        return "Septiembre";
      case 10:
        return "Octubre";
      case 11:
        return "Noviembre";
      case 12:
        return "Diciembre";
      default:
        return "Enero";
    }
  }

  int obtenerMes(String month) {
    String mes = month.toLowerCase();
    switch (mes) {
      case "enero":
        return 1;
      case "febrero":
        return 2;
      case "marzo":
        return 3;
      case "abril":
        return 4;
      case "mayo":
        return 5;
      case "junio":
        return 6;
      case "julio":
        return 7;
      case "agosto":
        return 8;
      case "septiembre":
        return 9;
      case "octubre":
        return 10;
      case "noviembre":
        return 11;
      case "diciembre":
        return 12;
      default:
        return 1;
    }
  }

 

  /// Navigate to the given month.
  void _showMonth(DateTime month, {bool jump = false}) {
    final int monthPage = DateUtils.monthDelta(widget.config.firstDate, month);
    if (jump) {
      _pageController.jumpToPage(monthPage);
    } else {
      _pageController.animateToPage(
        monthPage,
        duration: _monthScrollDuration,
        curve: Curves.ease,
      );
    }
  }

  /// True if the earliest allowable month is displayed.
  bool get _isDisplayingFirstMonth {
    return !_currentMonth.isAfter(
      DateTime(widget.config.firstDate.year, widget.config.firstDate.month),
    );
  }

  /// True if the latest allowable month is displayed.
  bool get _isDisplayingLastMonth {
    return !_currentMonth.isBefore(
      DateTime(widget.config.lastDate.year, widget.config.lastDate.month),
    );
  }

  /// Handler for when the overall day grid obtains or loses focus.
  void _handleGridFocusChange(bool focused) {
    setState(() {
      if (focused && _focusedDay == null && widget.selectedDates.isNotEmpty) {
        if (DateUtils.isSameMonth(widget.selectedDates[0], _currentMonth)) {
          _focusedDay = widget.selectedDates[0];
        } else if (DateUtils.isSameMonth(
            widget.config.currentDate, _currentMonth)) {
          _focusedDay = _focusableDayForMonth(
              _currentMonth, widget.config.currentDate.day);
        } else {
          _focusedDay = _focusableDayForMonth(_currentMonth, 1);
        }
      }
    });
  }

  /// Move focus to the next element after the day grid.
  void _handleGridNextFocus(NextFocusIntent intent) {
    _dayGridFocus.requestFocus();
    _dayGridFocus.nextFocus();
  }

  /// Move focus to the previous element before the day grid.
  void _handleGridPreviousFocus(PreviousFocusIntent intent) {
    _dayGridFocus.requestFocus();
    _dayGridFocus.previousFocus();
  }

  /// Move the internal focus date in the direction of the given intent.
  ///
  /// This will attempt to move the focused day to the next selectable day in
  /// the given direction. If the new date is not in the current month, then
  /// the page view will be scrolled to show the new date's month.
  ///
  /// For horizontal directions, it will move forward or backward a day (depending
  /// on the current [TextDirection]). For vertical directions it will move up and
  /// down a week at a time.
  void _handleDirectionFocus(DirectionalFocusIntent intent) {
    setState(() {
      if (_focusedDay != null) {
        final nextDate = _nextDateInDirection(_focusedDay!, intent.direction);
        if (nextDate != null) {
          _focusedDay = nextDate;
          if (!DateUtils.isSameMonth(_focusedDay, _currentMonth)) {
            _showMonth(_focusedDay!);
          }
        }
      } else {
        _focusedDay ??= widget.initialMonth;
      }
    });
  }

  static const Map<TraversalDirection, int> _directionOffset =
      <TraversalDirection, int>{
    TraversalDirection.up: -DateTime.daysPerWeek,
    TraversalDirection.right: 1,
    TraversalDirection.down: DateTime.daysPerWeek,
    TraversalDirection.left: -1,
  };

  int _dayDirectionOffset(
      TraversalDirection traversalDirection, TextDirection textDirection) {
    // Swap left and right if the text direction if RTL
    if (textDirection == TextDirection.rtl) {
      if (traversalDirection == TraversalDirection.left) {
        traversalDirection = TraversalDirection.right;
      } else if (traversalDirection == TraversalDirection.right) {
        traversalDirection = TraversalDirection.left;
      }
    }
    return _directionOffset[traversalDirection]!;
  }

  DateTime? _nextDateInDirection(DateTime date, TraversalDirection direction) {
    final TextDirection textDirection = Directionality.of(context);
    DateTime nextDate = DateUtils.addDaysToDate(
        date, _dayDirectionOffset(direction, textDirection));
    while (!nextDate.isBefore(widget.config.firstDate) &&
        !nextDate.isAfter(widget.config.lastDate)) {
      if (_isSelectable(nextDate)) {
        return nextDate;
      }
      nextDate = DateUtils.addDaysToDate(
          nextDate, _dayDirectionOffset(direction, textDirection));
    }
    return null;
  }

  bool _isSelectable(DateTime date) {
    return widget.config.selectableDayPredicate?.call(date) ?? true;
  }

  Widget _buildItems(BuildContext context, int index) {
    final DateTime month =
        DateUtils.addMonthsToMonthDate(widget.config.firstDate, index);
    return _DayPicker(
      key: ValueKey<DateTime>(month),
      selectedDates: (widget.selectedDates..removeWhere((d) => d == null))
          .cast<DateTime>(),
      onChanged: _handleDateSelected,
      config: widget.config,
      displayedMonth: month,
    );
  }

  //  List<int> anios
  // final List<int> anios = [2024];
  // var dropAnio = DateTime.now().year;

  void cargarAnios(){
    for (var i = 1900; i <= DateTime.now().year; i++) {
      anios.add(i);
    }
  }

  @override
  Widget build(BuildContext context) {
    final Color controlColor =
        Theme.of(context).colorScheme.onSurface.withOpacity(0.60);
    dropdownValue = obtenerIntMes(_currentMonth.month);
    if (anios.length<10) {
      cargarAnios();
    }
    
    return Semantics(
      child: Column(
        children: <Widget>[
          Container(
            padding: widget.config.centerAlignModePicker != true
                ? const EdgeInsetsDirectional.only(start: 16, end: 4)
                : const EdgeInsetsDirectional.only(start: 8, end: 8),
            height: (widget.config.controlsHeight ?? _subHeaderHeight),
            child: Row(
              children: <Widget>[
                if (widget.config.centerAlignModePicker != true) const Spacer(),
                Container(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.all(Radius.circular(61.6)),
                    color: Colors.white,
                  ),
                  child: IconButton(
                    iconSize: 30,
                    icon: widget.config.lastMonthIcon ??
                        const Icon(Icons.chevron_left),
                    color: Colors.black,
                    tooltip: _isDisplayingFirstMonth
                        ? null
                        : _localizations.previousMonthTooltip,
                    onPressed:
                        _isDisplayingFirstMonth ? null : _handlePreviousMonth,
                  ),
                ),
                // ElevatedButton(
                //   onPressed: () {
                //     _showMonth(DateTime(2000, 11), jump: true);
                //   },
                //   child: const Text('Enabled'),
                // ),
                const Spacer(),
                Container(
                  // width: 90,
                  // alignment: Alignment.center,
                  // padding: const EdgeInsets.all(10),
                  // alignment: Alignment.center,
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.all(Radius.circular(10)),
                    color: Colors.white,
                  ),
                  child: DropdownButton<String>(
                    // itemHeight: ,
                    alignment: Alignment.center,
                    underline: Container(
                      height: 0,
                      color: Colors.transparent,
                    ),
                    borderRadius: const BorderRadius.all(Radius.circular(5)),
                    dropdownColor: Colors.white,
                    value: dropdownValue,
                    focusColor: Colors.transparent,
                    icon: const Icon(
                      Icons.arrow_drop_down,
                      color: Color.fromRGBO(150, 198, 15, 1),
                      size: 30,
                    ),
                    style: const TextStyle(
                      color: Colors.black,
                      fontWeight: FontWeight.w800
                    ),
                    onChanged: (String? value) {
                      dropdownValue = value!;
                      int year = _currentMonth.year;
                      _showMonth(DateTime(year, obtenerMes(value)), jump: true);
                      // This is called when the user selects an item.
                      // setState(() {
                      //   dropdownValue = value!;
                      //   _showMonth(DateTime(2000, 11), jump: true);
                      //   // print(dropdownValue);
                      //   // _singleDatePickerValueWithDefaultValue = [
                      //   //   DateTime(1970, 3)
                      //   // ];
                      //   // widget.onChange!();
                      //   // print(_singleDatePickerValueWithDefaultValue);
                      // });
                    },
                    items: list.map<DropdownMenuItem<String>>((String value) {
                      return DropdownMenuItem<String>(
                        value: value,
                        child: Text(value),
                      );
                    }).toList(),
                  ),
                ),
                const SizedBox(width: 10,),
                Container(
                  // width: 70,
                   alignment: Alignment.center,
                  padding: const EdgeInsets.all(10),
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.all(Radius.circular(10)),
                    color: Colors.white,
                  ),
                  child: DropdownButton<int>(
                    // padding: EdgeInsets.all(10),
                    alignment: Alignment.center,
                    underline: Container(
                      height: 0,
                      color: Colors.transparent,
                    ),
                    borderRadius: const BorderRadius.all(Radius.circular(5)),
                    dropdownColor: Colors.white,
                    value: dropAnio,
                    focusColor: Colors.transparent,
                    icon: const Icon(
                      Icons.arrow_drop_down,
                      color: Color.fromRGBO(150, 198, 15, 1),
                      // size: 30,
                    ),
                    style: const TextStyle(
                      color: Colors.black,
                       fontWeight: FontWeight.w800
                    ),
                    onChanged: (int? value) {
                      // cargarAnios();
                      setState(() {
                        dropAnio = value!;
                      });
                      // print(value);
                       // dropAnio = value!;
                      _showMonth(DateTime(value!, _currentMonth.month), jump: true);
                      // dropAnio = value!;
                      // print("ERROR");
                      // print(value);

                      // int year = dropAnio;
                      // _showMonth(DateTime(year), jump: true);
                      // This is called when the user selects an item.
                      // setState(() {
                      //   dropdownValue = value!;
                      //   _showMonth(DateTime(2000, 11), jump: true);
                      //   // print(dropdownValue);
                      //   // _singleDatePickerValueWithDefaultValue = [
                      //   //   DateTime(1970, 3)
                      //   // ];
                      //   // widget.onChange!();
                      //   // print(_singleDatePickerValueWithDefaultValue);
                      // });
                    },
                    items: anios.map<DropdownMenuItem<int>>((int value) {
                      return DropdownMenuItem<int>(
                        value: value,
                        child: Text(value.toString()),
                      );
                    }).toList(),
                  ),
                ),
                // Spacer(),
                // DropdownButton<String>(
                //   dropdownColor: Colors.white,
                //   value: dropdownValue,
                //   icon: const Icon(
                //     Icons.arrow_drop_down,
                //     color: Color.fromRGBO(150, 198, 15, 1),
                //     size: 30,
                //   ),
                //   elevation: 16,
                //   style: const TextStyle(
                //     color: Colors.black,
                //   ),
                //   onChanged: (String? value) {
                //     dropdownValue = value!;
                //     int year = _currentMonth.year;
                //     _showMonth(DateTime(year, obtenerMes(value!)), jump: true);
                //     // This is called when the user selects an item.
                //     // setState(() {
                //     //   dropdownValue = value!;
                //     //   _showMonth(DateTime(2000, 11), jump: true);
                //     //   // print(dropdownValue);
                //     //   // _singleDatePickerValueWithDefaultValue = [
                //     //   //   DateTime(1970, 3)
                //     //   // ];
                //     //   // widget.onChange!();
                //     //   // print(_singleDatePickerValueWithDefaultValue);
                //     // });
                //   },
                //   items: list.map<DropdownMenuItem<String>>((String value) {
                //     return DropdownMenuItem<String>(
                //       value: value,
                //       child: Text(value),
                //     );
                //   }).toList(),
                // ),
                if (widget.config.centerAlignModePicker == true) const Spacer(),
                Container(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.all(Radius.circular(61.6)),
                    color: Colors.white,
                  ),
                  child: IconButton(
                    iconSize: 30,
                    icon: widget.config.nextMonthIcon ??
                        const Icon(Icons.chevron_right),
                    color: Colors.black,
                    tooltip: _isDisplayingLastMonth
                        ? null
                        : _localizations.nextMonthTooltip,
                    onPressed: _isDisplayingLastMonth ? null : _handleNextMonth,
                  ),
                ),
              ],
            ),
          ),
          Expanded(
            child: FocusableActionDetector(
              shortcuts: _shortcutMap,
              actions: _actionMap,
              focusNode: _dayGridFocus,
              onFocusChange: _handleGridFocusChange,
              child: _FocusedDate(
                date: _dayGridFocus.hasFocus ? _focusedDay : null,
                child: PageView.builder(
                  key: _pageViewKey,
                  controller: _pageController,
                  itemBuilder: _buildItems,
                  itemCount: DateUtils.monthDelta(
                          widget.config.firstDate, widget.config.lastDate) +
                      1,
                  onPageChanged: _handleMonthPageChanged,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
